searchData={"content_type":"text/markdown","items":[{"doc":"Documentation for `ExCrawlzy`.","ref":"ExCrawlzy.html","title":"ExCrawlzy","type":"module"},{"doc":"Request link and returns the raw content.","ref":"ExCrawlzy.html#crawl/2","title":"ExCrawlzy.crawl/2","type":"function"},{"doc":"iex> ExCrawlzy.crawl(\"http://some.site\")\n    {:ok, \"          the title                  the body               inner field                      inner second field                   2023                               this field exist                               \"}","ref":"ExCrawlzy.html#crawl/2-examples","title":"Examples - ExCrawlzy.crawl/2","type":"function"},{"doc":"Request link and returns the raw content.","ref":"ExCrawlzy.html#parse/2","title":"ExCrawlzy.parse/2","type":"function"},{"doc":"iex> raw_content = \"   the title    the body   \"\n    iex> ExCrawlzy.parse(%{body: {\"#the_body\", :text}}, raw_content)\n    {:ok, %{body: \"the body\"}}","ref":"ExCrawlzy.html#parse/2-examples","title":"Examples - ExCrawlzy.parse/2","type":"function"},{"doc":"","ref":"ExCrawlzy.html#t:map_key/0","title":"ExCrawlzy.map_key/0","type":"type"},{"doc":"","ref":"ExCrawlzy.html#t:post_processing/0","title":"ExCrawlzy.post_processing/0","type":"type"},{"doc":"","ref":"ExCrawlzy.html#t:result/0","title":"ExCrawlzy.result/0","type":"type"},{"doc":"","ref":"ExCrawlzy.html#t:selector_tuple/0","title":"ExCrawlzy.selector_tuple/0","type":"type"},{"doc":"Simulation of real clients from different browsers in order to avoid bot detection","ref":"ExCrawlzy.BrowserClients.html","title":"ExCrawlzy.BrowserClients","type":"module"},{"doc":"","ref":"ExCrawlzy.BrowserClients.html#clients/0","title":"ExCrawlzy.BrowserClients.clients/0","type":"function"},{"doc":"Creates a client adding the headers in the config","ref":"ExCrawlzy.BrowserClients.html#create_client/1","title":"ExCrawlzy.BrowserClients.create_client/1","type":"function"},{"doc":"","ref":"ExCrawlzy.BrowserClients.html#t:client/0","title":"ExCrawlzy.BrowserClients.client/0","type":"type"},{"doc":"","ref":"ExCrawlzy.BrowserClients.html#t:header/0","title":"ExCrawlzy.BrowserClients.header/0","type":"type"},{"doc":"Interface to implement a new ExCrawlzy client","ref":"ExCrawlzy.Client.Crawler.html","title":"ExCrawlzy.Client.Crawler","type":"module"},{"doc":"","ref":"ExCrawlzy.Client.Crawler.Interface.html","title":"ExCrawlzy.Client.Crawler.Interface","type":"behaviour"},{"doc":"","ref":"ExCrawlzy.Client.Crawler.Interface.html#c:crawl/1","title":"ExCrawlzy.Client.Crawler.Interface.crawl/1","type":"callback"},{"doc":"Module to add more browser clients to the predefined client module\n\nadd to the module the `use` and use the macro `add_browser_client/1`\n\n```elixir\ndefmodule Some.Module do\n  use ExCrawlzy.Client.Handlers.BrowserClients\n\n  add_browser_client([\n    {\"referer\", \"https://your_site.com\"},\n    {\"user-agent\", \"Custom User Agent\"}\n  ])\n  ...\n```\n\nand adds the function `fields/0` to the module\n\n```elixir\n> Some.Module.clients()\n[\n  %Tesla.Client{\n    fun: nil,\n    pre: [\n      {Tesla.Middleware.Compression, :call, [[format: \"gzip\"]]},\n      {Tesla.Middleware.Headers, :call,\n       [\n         [\n           {\"referer\", \"https://your_site.com\"},\n           {\"user-agent\", \"Custom User Agent\"}\n         ]\n       ]}\n    ],\n    post: [],\n    adapter: nil\n  }\n]\n```","ref":"ExCrawlzy.Client.Handlers.BrowserClients.html","title":"ExCrawlzy.Client.Handlers.BrowserClients","type":"module"},{"doc":"Add a new browser client to the module sending headers\n\nExample headers\n\n```\n{\"referer\", \"https://your_site.com\"},\n{\"user-agent\", \"Custom User Agent\"}\n```","ref":"ExCrawlzy.Client.Handlers.BrowserClients.html#add_browser_client/1","title":"ExCrawlzy.Client.Handlers.BrowserClients.add_browser_client/1","type":"macro"},{"doc":"Internal module to handle json fields, mainly helps to create modules with defined keys on compile\n\nadd to the module the `use` and use the macro `add_field/3`\n\n```elixir\ndefmodule Some.Module do\n  use ExCrawlzy.Client.Handlers.Fields\n\n  add_field(:name, \"a.mr-1 span.repo\", :text)\n  ...\n```\n\nand adds the function `fields/0` to the module\n\n```elixir\n> Some.Module.fields()\n%{name: {\"a.mr-1 span.repo\", :text}}\n```","ref":"ExCrawlzy.Client.Handlers.Fields.html","title":"ExCrawlzy.Client.Handlers.Fields","type":"module"},{"doc":"Add the params in the inner data, all are saved on the module and you can get the whole data using the function `fields/0`","ref":"ExCrawlzy.Client.Handlers.Fields.html#add_field/3","title":"ExCrawlzy.Client.Handlers.Fields.add_field/3","type":"macro"},{"doc":"Internal module to handle json fields, mainly helps to create modules with defined list selector match on compile\n\nadd to the module the `use` and use the macro `list_selector/1` to specify the multiple selector and `list_size/1` to\nspecify the amount of elements to take\n\n```elixir\ndefmodule Some.Module do\n  use ExCrawlzy.Client.Handlers.List\n\n  list_size(5)\n  list_selector(\"div.element-1/3\")\n  ...\n```\n\nand adds the functions `list_size/0` and `get_list_selector_selector/0` to the module\n\n```elixir\n> Some.Module.list_size()\n5\n> Some.Module.get_list_selector_selector()\n\"div.element-1/3\"\n```","ref":"ExCrawlzy.Client.Handlers.List.html","title":"ExCrawlzy.Client.Handlers.List","type":"module"},{"doc":"Add the a selector to multiple items","ref":"ExCrawlzy.Client.Handlers.List.html#list_selector/1","title":"ExCrawlzy.Client.Handlers.List.list_selector/1","type":"macro"},{"doc":"To specify the number of elements to take on the crawl","ref":"ExCrawlzy.Client.Handlers.List.html#list_size/1","title":"ExCrawlzy.Client.Handlers.List.list_size/1","type":"macro"},{"doc":"Module that helps to create clients to crawl some sites and extract the data as json/map\n\nThe point its create other module that uses this module and add fields using the macro `add_field/3`\n\nThen just use the client function `crawl/1` with your url and will crawl and parse the data and return the json/map\n\n```elixir\ndefmodule ExampleCrawler do\n  use ExCrawlzy.Client.Json\n\n  add_field(:title, \"head title\", :text)\n  add_field(:field_1, \"div#some_id\", :text_alt)\n  add_field(:link, \"a#external_reference\", :link)\n  add_field(:open_new_tab, \"a#external_reference\", :external_target)\n\n  def text_alt(sub_doc) do\n    ExCrawlzy.Utils.text(sub_doc)\n  end\n\n  def external_target(sub_doc) do\n    ExCrawlzy.Utils.props(\"target\", sub_doc) == \"_blank\"\n  end\nend\n\nExampleCrawler.crawl(\"http://some.site\")\n```","ref":"ExCrawlzy.Client.Json.html","title":"ExCrawlzy.Client.Json","type":"module"},{"doc":"Module that helps to create clients to crawl some sites and extract the data as list of json/map\n\nThe point its create other module that uses this module, add the list selector with `list_selector/1` and add fields\nusing the macro `add_field/3`\n\nBy default takes 20 elements but you can customize using `list_size/1`\n\nThen just use the client function `crawl/1` with your url and will crawl and parse the data and return the list of json/map\n\nExample:\n\n```elixir\ndefmodule GithubProfilePinnedRepos do\n  use ExCrawlzy.Client.JsonList\n\n  add_browser_client([\n    {\"referer\", \"https://your_site.com\"},\n    {\"user-agent\", \"Custom User Agent\"}\n  ])\n\n  list_selector(\"div.pinned-item-list-item\")\n  add_field(:name, \"a.mr-1 span.repo\", :text)\n  add_field(:link, \"a.mr-1\", :link)\n  add_field(:access, \"span.Label\", :text)\n  add_field(:description, \"p.pinned-item-desc\", :text)\n  add_field(:language, \"span.d-inline-block span[itemprop=\"programmingLanguage\"]\", :text)\n\n  def link(doc) do\n    path = ExCrawlzy.Utils.props(\"href\", doc)\n    \"https://github.com#{path}\"\n  end\nend\n\nGithubProfilePinnedRepos.crawl(\"https://github.com/nicolkill\")\n```","ref":"ExCrawlzy.Client.JsonList.html","title":"ExCrawlzy.Client.JsonList","type":"module"},{"doc":"Utilities for the whole library\n\nhelping to parsing data and extract from css docs","ref":"ExCrawlzy.Utils.html","title":"ExCrawlzy.Utils","type":"module"},{"doc":"Transform binary to readable strings\n\n    iex> ExCrawlzy.Utils.binary_to_string( >)\n    \"some string\"","ref":"ExCrawlzy.Utils.html#binary_to_string/1","title":"ExCrawlzy.Utils.binary_to_string/1","type":"function"},{"doc":"Returns if some element exist\n\n  Examples:\n\n      iex> ExCrawlzy.Utils.exist([{\"h1\", [{\"class\", \"some_class\"}], [\"My text inside a h1\"]}])\n      true\n\n      iex> ExCrawlzy.Utils.exist([])\n      false","ref":"ExCrawlzy.Utils.html#exist/1","title":"ExCrawlzy.Utils.exist/1","type":"function"},{"doc":"","ref":"ExCrawlzy.Utils.html#iframe/1","title":"ExCrawlzy.Utils.iframe/1","type":"function"},{"doc":"","ref":"ExCrawlzy.Utils.html#img/1","title":"ExCrawlzy.Utils.img/1","type":"function"},{"doc":"","ref":"ExCrawlzy.Utils.html#link/1","title":"ExCrawlzy.Utils.link/1","type":"function"},{"doc":"Extract specific data from html element props\n\n  For example on a simple link ` My Link ` you can extract just the data of the `href` prop\n\n  Examples:\n\n    iex> ExCrawlzy.Utils.props(\"href\", [{\"a\", [{\"href\", \"http://site.example\"}], []}])\n    \"http://site.example\"\n    iex> ExCrawlzy.Utils.props(\"target\", [{\"span\", [{\"target\", \"some_value\"}], []}])\n    \"some_value\"","ref":"ExCrawlzy.Utils.html#props/2","title":"ExCrawlzy.Utils.props/2","type":"function"},{"doc":"Extract specific data based on html inner element as text, works great for html simple elements like `span`, `p`, `h1` and even more\n\n  For example on a simple link ` My text inside a h1 ` you can extract the text inside the element\n\n  Examples:\n\n      iex> ExCrawlzy.Utils.text([{\"h1\", [{\"class\", \"some_class\"}], [\"My text inside a h1\"]}])\n      \"My text inside a h1\"","ref":"ExCrawlzy.Utils.html#text/1","title":"ExCrawlzy.Utils.text/1","type":"function"},{"doc":"","ref":"readme.html","title":"README","type":"extras"},{"doc":"Another crawling library but with more than just crawl\n\nYou can crawl sites and transform content to json/map using CSS selectors with no other libraries, utilities included\nand more than a simple integration, you can transform a simple site to json with fields like lists or another sub-json\nstructures","ref":"readme.html#excrawlzy","title":"ExCrawlzy - README","type":"extras"},{"doc":"If [available in Hex](https://hexdocs.pm/ex_crawlzy), the package can be installed\nby adding `ex_crawlzy` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_crawlzy, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"Just use the function [`ExCrawlzy.crawl/1`](https://hexdocs.pm/ex_crawlzy/ExCrawlzy.html#crawl/1) to\ncrawl and [`ExCrawlzy.parse/2`](https://hexdocs.pm/ex_crawlzy/ExCrawlzy.html#parse/2) to parse to json\n\n##","ref":"readme.html#usage","title":"Usage - README","type":"extras"},{"doc":"```elixir\nsite = \"https://example.site\"\n\nfields = %{\n  # shortcut for use a function from ExCrawlzy.Utils\n  body: {\"div#the_body\", :text}\n#  module/function way\n#  body: {\"div#the_body\", {ExCrawlzy.Utils, :text}}\n#  body: {\"div#the_body\", fn content -> \n#   ExCrawlzy.Utils.text(content)\n#  end}\n}\n\n{:ok, content} = ExCrawlzy.crawl(site)\n{:ok, %{body: body}} = ExCrawlzy.parse(fields, content)\n```\n\n##","ref":"readme.html#basic-usage","title":"Basic usage - README","type":"extras"},{"doc":"You can create a module pre-configured with key, selector and processing functions and just call using the function\n`crawl/1` inside the same module\n\n```elixir\ndefmodule ExampleCrawler do\n  use ExCrawlzy.Client.Json\n  \n  add_field(:title, \"head title\", :text)\n  add_field(:body, \"div#the_body\", :text)\n  add_field(:inner_field, \"div#the_body div#inner_field\", :text)\n  add_field(:inner_second_field, \"div#inner_second_field\", :text_alt)\n  add_field(:number, \"div#the_number\", :text)\n  add_field(:exist, \"div#the_body div#exist\", :exist)\n  add_field(:not_exist, \"div#the_body div#not_exist\", :exist)\n  add_field(:link, \"a.link_class\", :link)\n  add_field(:img, \"img.img_class\", :img)\n\n  def text_alt(sub_doc) do\n    ExCrawlzy.Utils.text(sub_doc)\n  end\nend\n\nsite = \"https://example.site\"\n\n{:ok, data} = ExampleCrawler.crawl(site)\n```\n\n##","ref":"readme.html#using-client","title":"Using Client - README","type":"extras"},{"doc":"You can create a client that parses multiple elements from html using css selectors\n\nUsing `list_selector/1` you can define the selector that all elements matches, the next is define as the client \n`ExCrawlzy.Client.Json` and this are the inner elements\n\n```elixir\ndefmodule ExampleCrawlerList do\n  use ExCrawlzy.Client.JsonList\n\n  list_size(2)\n  list_selector(\"div.possible_value\")\n  add_field(:field_1, \"div.field_1\", :text)\n  add_field(:field_2, \"div.field_2\", :text)\nend\n\nsite = \"https://example_list.site\"\n\n{:ok, data} = ExampleCrawlerList.crawl(site)\n```\n\n##","ref":"readme.html#list-of-elements","title":"List of elements - README","type":"extras"},{"doc":"```elixir\ndefmodule GithubProfilePinnedRepos do\n  use ExCrawlzy.Client.JsonList\n\n  list_selector(\"div.pinned-item-list-item\")\n  add_field(:name, \"a.mr-1 span.repo\", :text)\n  add_field(:link, \"a.mr-1\", :link)\n  add_field(:access, \"span.Label\", :text)\n  add_field(:description, \"p.pinned-item-desc\", :text)\n  add_field(:language, \"span.d-inline-block span[itemprop=\\\"programmingLanguage\\\"]\", :text)\n\n  def link(doc) do\n    path = ExCrawlzy.Utils.props(\"href\", doc)\n    \"https://github.com#{path}\"\n  end\nend\n\nsite = \"https://github.com/nicolkill\"\n\n{\n  :ok, \n  [\n    %{\n      access: \"Public\",\n      description: \"An API Prototype Platform\",\n      link: \"https://github.com/nicolkill/dbb\",\n      name: \"dbb\",\n      language: \"Elixir\"\n    },\n    %{\n      access: \"Public\",\n      description: \"JSON Schema verifier in Elixir\",\n      link: \"https://github.com/nicolkill/map_schema_validator\",\n      name: \"map_schema_validator\",\n      language: \"Elixir\"\n    },\n    %{\n      access: \"Public\",\n      description: \"\",\n      link: \"https://github.com/nicolkill/ex_crawlzy\",\n      name: \"ex_crawlzy\",\n      language: \"Elixir\"\n    }\n  ]\n} == ExampleCrawlerList.crawl(site)\n```","ref":"readme.html#a-good-example","title":"A good example - README","type":"extras"},{"doc":"You can define you own browser clients on the requests, just use the function `add_browser_client/1` and your headers\non this shape `[{\"header_name\", \"header value\"}]`\n\n> Add your own browser clients will replace the predefined ones\n\n```elixir\nsite = \"https://example.site\"\n\nfields = %{\n  body: {\"div#the_body\", :text}\n}\n\nclients = [\n  [\n    {\"referer\", \"https://your_site.com\"},\n    {\"user-agent\", \"Custom User Agent\"}\n  ]\n]\n\n{:ok, content} = ExCrawlzy.crawl(site, clients)\n{:ok, %{body: body}} = ExCrawlzy.parse(fields, content)\n\ndefmodule ExampleCrawlerList do\n  use ExCrawlzy.Client.JsonList\n\n  add_browser_client([\n    {\"referer\", \"https://your_site.com\"},\n    {\"user-agent\", \"Custom User Agent\"}\n  ])\n  list_size(2)\n  list_selector(\"div.possible_value\")\n  add_field(:field_1, \"div.field_1\", :text)\n  add_field(:field_2, \"div.field_2\", :text)\nend\n\nsite = \"https://example_list.site\"\n\n{:ok, data} = ExampleCrawlerList.crawl(site)\n```","ref":"readme.html#add-clients","title":"Add clients - README","type":"extras"}]}